/** 
 *******************************************************************************
 * @file    safety_uart.h
 * @brief   This file provides all the functions prototypes for Safety Library - UART.
 * @version V0.1.0.1
 * $Date:: 2018-03-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef _SAFETY_UART_H
#define _SAFETY_UART_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_safety_def.h"
#include "txz_uart.h"

/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup UARTTest UART Test
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Exported_define UARTTest Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group UARTTest_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Exported_define UARTTest Exported Define
 *  @{
 */

// UART ID for SAFETY UART
// can be any value but use thesae pre-assigned values for now
typedef enum
{
    SAFETY_UART0_ID = 100,
    SAFETY_UART1_ID = 101,
    SAFETY_UART2_ID = 102,
    SAFETY_UART3_ID = 103,
    
    SAFETY_NOT_USED_ID = 0
      
 } SafetyUartID;


/**
 *  @}
 */ /* End of group UARTTest_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Exported_typedef UARTTest Exported Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group UARTTest_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Exported_functions UARTTest Exported Functions
 *  @{
 */
  
// IRQ handler for safety_UART_loopback  
// return immediately if safety_UART_loopback not running
void safety_UART_hook_tx(SafetyUartID id);
void safety_UART_hook_rx(SafetyUartID id);
void safety_UART_hook_err(SafetyUartID id);
  
bool safety_UART_loopback(SafetyUartID id, uint32_t start, uint32_t length);

/**
 *  @}
 */ /* End of group UARTTest_Exported_functions */

/**
 *  @}
 */ /* End of group UARTTest */

/**
 *  @}
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* _SAFETY_UART_H */


