/**
 *******************************************************************************
 * @file    safety_analog_in.c
 * @brief   This file provides API functions for Safety Library - ANALOG IN.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "safety_analog_in.h"

// gpio driver definitions
#include "txz_gpio.h"  

// adc driver definitions
#include "txz_adc_ex.h"     // this will also include "adc.h"
#include "txz_adc_ch.h"
      
// use bsp functions
#include "bsp.h"  

   
/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup ANALOG_IN_Test ANALOGIN Test
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ANALOG_IN_Test_Private_define ANALOG_IN_Test Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group ANALOG_IN_Test_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ANALOG_IN_Test_Private_define ANALOG_IN_Test Private Define
 *  @{
 */
    
  
/**
 *  @}
 */ /* End of group ANALOG_IN_Test_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ANALOG_IN_Test_Private_define ANALOG_IN_Test Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ANALOG_IN_Test_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ANALOG_IN_Test_Private_typedef ANALOG_IN_Test Private Typedef
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_analog1[29];
        char DEBUGMSG_analog2[40];
        char DEBUGMSG_analog3[34];
        char DEBUGMSG_analog4[33];
        char DEBUGMSG_analog5[33];
        char DEBUGMSG_analog6[26];
        char DEBUGMSG_analog7[36];
        char DEBUGMSG_analog8[58];
        char DEBUGMSG_analog9[34];
        char DEBUGMSG_analog10[49];
        char DEBUGMSG_analog11[40];
        char DEBUGMSG_analog12[37];
        char DEBUGMSG_analog13[59];
        char DEBUGMSG_analog14[50];
        char DEBUGMSG_analog15[18];
        char DEBUGMSG_analog16[60];
        char DEBUGMSG_analog17[59];
        char DEBUGMSG_analog18[50];
        char DEBUGMSG_analog19[18];
        char DEBUGMSG_analog20[58];
        char DEBUGMSG_analog21[59];
        char DEBUGMSG_analog22[50];
        char DEBUGMSG_analog23[15];
        char DEBUGMSG_analog24[58];
    } debugmsg_t;
#endif

/**
 *  @}
 */ /* End of group ANALOG_IN_Test_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t analog;
    
    char DEBUGMSG_analog_temp[24][60] = {{"adc_ex_initialize: NULL ptr."}, 
                                      {"get_adc_value: start conversion failed."}, 
                                      {"get_adc_value: get status failed."}, 
                                      {"get_adc_value: get value failed."}, 
                                      {"get_adc_value: value = 0xp_value"},
                                      {"safety_AIN_PE4: NULL ptr."},
                                      {"safety_AIN_PE4: adc_ex_init failed."},
                                      {"safety_AIN_PE4: adc_ex_channel_setting(CH=ADC_CH) failed."},
                                      {"safety_AIN_PE4: setup PE4 failed."},
                                      {"safety_AIN_PE4: get_adc_value(CH=ADC_CH) failed."},
                                      {"safety_AIN_PE4: stop conversion failed."},
                                      {"safety_AnalogIn: adc_ex_init failed."},
                                      {"safety_AnalogIn: adc_ex_channel_setting(CH=ADC_CH) failed."},
                                      {"safety_AnalogIn: get_adc_value(CH=ADC_CH) failed."},
                                      {"Lo Outout=0xvalue"},
                                      {"safety_AnalogIn: PULLDOWN test: value 0xvalue out of range."},
                                      {"safety_AnalogIn: adc_ex_channel_setting(CH=ADC_CH) failed."},
                                      {"safety_AnalogIn: get_adc_value(CH=ADC_CH) failed."},
                                      {"Hi Output=0xvalue"},
                                      {"safety_AnalogIn: PULLUP test: value 0xvalue out of range."},
                                      {"safety_AnalogIn: adc_ex_channel_setting(CH=ADC_CH) failed."},
                                      {"safety_AnalogIn: get_adc_value(CH=ADC_CH) failed."},
                                      {"MIDVOL=0xvalue"},
                                      {"safety_AnalogIn: MIDVOL test: value 0xvalue out of range."}};

    
#endif

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ANALOG_IN_Test_Private_fuctions ANALOG_IN_Test Private Fuctions
 *  @{
 */
static void adc_ex_initstruct(adc_ex_t *p_adc_ex);
static bool get_adc_value(adc_ex_t *p_adc_ex, uint32_t ch, uint32_t *p_value);

 // initialize instance structure
static void adc_ex_initstruct(adc_ex_t *p_adc_ex)
{
    if (p_adc_ex == SAFETY_NULL) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("adc_ex_initialize: NULL ptr.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog1,&DEBUGMSG_analog_temp[0][0]);
#endif
     return;
    }

    /* Register Allocation */
    p_adc_ex->super.p_instance       = TSB_ADA;

    /*--- Initialize ---*/
    p_adc_ex->super.init.id            = (uint32_t)p_adc_ex;
    p_adc_ex->super.init.convert_time  = ADC_CONVERT_TIME_0_50_AVDD_4_5;
    p_adc_ex->super.init.refbsel       = ADC_REFBSEL_AIN;
    p_adc_ex->super.init.rcut          = ADC_RCUT_NORMAL;
    p_adc_ex->super.handler            = SAFETY_NULL;
    
    // all channel instance info are cleard by "clear_ch_instance_info()" via "adc_ex_init()"
    // no need following
    //int i;
    //for (i = 0; i < ADC_NUM_MAX; i++)
    //    p_adc_ex->super.info.ch[i].init.type = ADC_CONVERSION_DISABLE;

    /* EX Class Initialize */
    p_adc_ex->handler.single     = SAFETY_NULL;
    p_adc_ex->handler.continuity = SAFETY_NULL;
    p_adc_ex->handler.trigger    = SAFETY_NULL;      
}

// start ADC and wait for Complete. return ADC value
static bool get_adc_value(adc_ex_t *p_adc_ex, uint32_t ch, uint32_t *p_value)
{
    uint32_t status;
    
    // start conversion
    if (adc_ex_start(p_adc_ex) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("get_adc_value: start conversion failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog2,&DEBUGMSG_analog_temp[1][0]);
#endif
      return false;
    }

    // wait for Single Conversion complete
    do {
      
      if (adc_ex_get_status(p_adc_ex, &status) != TXZ_SUCCESS) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("get_adc_value: get status failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog3,&DEBUGMSG_analog_temp[2][0]);
#endif
        return false;
      }
      
    } while ((status & ADC_SGL_STATUS_MASK) == ADC_SGL_STATUS_RUNNING);
    
    // single conversion done
    // read adc result for ch
    if (adc_ex_channel_get_value(p_adc_ex, ch, p_value) != TXZ_SUCCESS) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("get_adc_value: get value failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog4,&DEBUGMSG_analog_temp[3][0]);
#endif
        return false;
    }    
    
    // this is for DEBUG
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("get_adc_value: value = 0x%x\r\n", *p_value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(analog.DEBUGMSG_analog5,&DEBUGMSG_analog_temp[4][0]);
#endif

    return true;
}

/**
 *  @}
 */ /* End of group ANALOG_IN_Test_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ANALOG_IN_Test_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     ADC Convert using PE4/AINA14 and ADC CH 14
  * @retval    0 :failed, 1 :success
  * @note      connected to VR1 slider volume in TMPM4K4-SBK
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/

bool safety_AIN_PE4(uint32_t *p_value)
{
    // use static memory area to store ADC info
    static adc_ex_t adc_ex_local;
    adc_ex_t *p_adc_ex = &adc_ex_local;
    
    // use ADC CH 14 for conversion
    static const uint32_t ADC_CH = 14;
    
    // ADC channel setting info (no IRQ, Single, AIN14)
    static adc_ex_channel_setting_t param = { ADC_INT_DISABLE, ADC_CONVERSION_SGL, 14 };

    gpio_t *p_gpio = bsp_get_gpio_instance_address();
    
    TXZ_Result temp;
    
    // check parameter
    if (p_value == SAFETY_NULL) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AIN_PE4: NULL ptr.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog6,&DEBUGMSG_analog_temp[5][0]);
#endif
      return false;
    }
        
    // setup adc_ex info structure and initialize
    adc_ex_initstruct(p_adc_ex);
    if (adc_ex_init(p_adc_ex) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AIN_PE4: adc_ex_init failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog7,&DEBUGMSG_analog_temp[6][0]);
#endif
      return false;
    }

    // setup ADC channel to use
    if (adc_ex_channel_setting(p_adc_ex, ADC_CH, &param) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AIN_PE4: adc_ex_channel_setting(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog8,&DEBUGMSG_analog_temp[7][0]);
#endif
      return false;
    }
    
    // setup PE4 for ADC
    // no need to switch function to use ADC
    // GPIO_Mode_DATA : don't care
    // GPIO_Mode_CR : off
    // GPIO_Mode_OD : off
    // GPIO_Mode_PUP : off
    // GPIO_Mode_PDN : off
    // GPIO_Mode_IE : off
    temp = gpio_write_bit(p_gpio, GPIO_PORT_E, GPIO_PORT_4, GPIO_Mode_CR, GPIO_PIN_RESET);
    temp |= gpio_write_bit(p_gpio, GPIO_PORT_E, GPIO_PORT_4, GPIO_Mode_OD, GPIO_PIN_RESET);
    temp |= gpio_write_bit(p_gpio, GPIO_PORT_E, GPIO_PORT_4, GPIO_Mode_PUP, GPIO_PIN_RESET);
    temp |= gpio_write_bit(p_gpio, GPIO_PORT_E, GPIO_PORT_4, GPIO_Mode_PDN, GPIO_PIN_RESET);
    temp |= gpio_write_bit(p_gpio, GPIO_PORT_E, GPIO_PORT_4, GPIO_Mode_IE, GPIO_PIN_RESET);
    if (temp != TXZ_SUCCESS) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AIN_PE4: setup PE4 failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog9,&DEBUGMSG_analog_temp[8][0]);
#endif
      return false;
    }
    
    // start ADC and get converted value
    if (!get_adc_value(p_adc_ex, ADC_CH, p_value)) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AIN_PE4: get_adc_value(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog10,&DEBUGMSG_analog_temp[9][0]);
#endif
      return false;
    }
        
    // finally, stop adc
    if (adc_ex_stop(p_adc_ex) != TXZ_SUCCESS) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AIN_PE4: stop conversion failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog11,&DEBUGMSG_analog_temp[10][0]);
#endif
      return false;
    }

    return true;        // get ADC success
}

/*--------------------------------------------------*/
/**
  * @brief     ADC In Test using PE0/AINA10 and ADC CH 10
  * @retval    none
  * @note      connected to VR1 slider volume in  TMPM4K4-SBK
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/

// threshold value, +- 10%
#define MAX_PULLDOWN    (0x19a) 	// 0.5V (5.0V x10%)
#define MIN_PULLDOWN    (0x0)	        // 0.0V

#define MAX_PULLUP    (0xfff)		// 5.0V
#define MIN_PULLUP    (0xe66)		// 4.5V (5.0V x90%)

#define MAX_MIDVOL    (0x439)		// 1.32V (1.20V +10%) 
#define MIN_MIDVOL    (0x374) 		// 1.08V (1.20V -10%) 


bool safety_AnalogIn(uint32_t *result_word)
{
    // use static memory area to store ADC info
    static adc_ex_t adc_ex_local;
    adc_ex_t *p_adc_ex = &adc_ex_local;
    
    // use ADC CH 10 for conversion
    static const uint32_t ADC_CH = 10;
    
    // ADC channel setting info (no IRQ, Single)
    // AIN ch (16/17/18) will be set just before use
    static adc_ex_channel_setting_t param = { ADC_INT_DISABLE, ADC_CONVERSION_SGL, 0};

    gpio_t *p_gpio = bsp_get_gpio_instance_address();
    
    uint32_t value, result;
    

    // clear result_word before start
    result = 0;
    if (result_word != SAFETY_NULL) {
      *result_word = 0;
    }
    
    // setup adc_ex info structure and initialize
    adc_ex_initstruct(p_adc_ex);
    if (adc_ex_init(p_adc_ex) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: adc_ex_init failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog12,&DEBUGMSG_analog_temp[11][0]);
#endif
      return false;
    }

    //
    // test case for VREFL (AIN17)
    //

    // select AIN17 for ADC CH 10
    param.ain = 17;
    if (adc_ex_channel_setting(p_adc_ex, ADC_CH, &param) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: adc_ex_channel_setting(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog13,&DEBUGMSG_analog_temp[12][0]);
#endif
      return false;
    }
        
    // start ADC and get value
    if (!get_adc_value(p_adc_ex, ADC_CH, &value)) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: get_adc_value(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog14,&DEBUGMSG_analog_temp[13][0]);
#endif
      return false;
    }
    
    // check return value
#if(MIN_PULLDOWN==0)
    if (value <= MAX_PULLDOWN)
#else
    if (MIN_PULLDOWN <= value && value <= MAX_PULLDOWN)
#endif
    {
      // the value is in range
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("Lo Outout=0x%x\r\n", value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog15,&DEBUGMSG_analog_temp[14][0]);
#endif
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: PULLDOWN test: value 0x%x out of range.\r\n", value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog16,&DEBUGMSG_analog_temp[15][0]);
#endif

      result = (1 << 0);       // set bit0 only
      goto error_return;
    }
    

    //
    // test case for VREFH (AIN16)
    //

    // select AIN16 for ADC CH 10
    param.ain = 16;
    if (adc_ex_channel_setting(p_adc_ex, ADC_CH, &param) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: adc_ex_channel_setting(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog17,&DEBUGMSG_analog_temp[16][0]);
#endif
      return false;
    }
            
    // start ADC and get value
    if (!get_adc_value(p_adc_ex, ADC_CH, &value)) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: get_adc_value(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog18,&DEBUGMSG_analog_temp[17][0]);
#endif
      return false;
    }

    // check return value
    if ((MIN_PULLUP <= value) && (value <= MAX_PULLUP)) {
      // the value is in range
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("Hi Output=0x%x\r\n", value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog19,&DEBUGMSG_analog_temp[18][0]);
#endif
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: PULLUP test: value 0x%x out of range.\r\n", value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog20,&DEBUGMSG_analog_temp[19][0]);
#endif

      result = (1 << 1);       // set bit1 only
      goto error_return;
    }

    
    //
    // test case for REGOUT1 (AIN18)
    //

    // select AIN18 for ADC CH 10
    param.ain = 18;
    if (adc_ex_channel_setting(p_adc_ex, ADC_CH, &param) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: adc_ex_channel_setting(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog21,&DEBUGMSG_analog_temp[20][0]);
#endif
      return false;
    }
    
    // start ADC
    if (!get_adc_value(p_adc_ex, ADC_CH, &value)) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: get_adc_value(CH=%d) failed.\r\n", ADC_CH));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog22,&DEBUGMSG_analog_temp[21][0]);
#endif
      return false;
    }
    
    // check return value
    if ((MIN_MIDVOL <= value) && (value <= MAX_MIDVOL)) {
      // the value is in range
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("MIDVOL=0x%x\r\n", value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog23,&DEBUGMSG_analog_temp[22][0]);
#endif
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_AnalogIn: MIDVOL test: value 0x%x out of range.\r\n", value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(analog.DEBUGMSG_analog24,&DEBUGMSG_analog_temp[23][0]);
#endif

      result = (1 << 2);       // set bit2 only
      goto error_return;
    }
    
    //
    // TEST all success
    //

    // result == 0
    // nothing to do special here.
    
error_return:
  
    // in case failed ???
    if (result != 0) {
      // update result word
      if (result_word != SAFETY_NULL) {
        *result_word = result;
      }
    }

    // return final result
    return (result == 0) ? true : false;
}


/**
 *  @}
 */ /* End of group ANALOG_IN_Test_Exported_functions */

/**
 *  @} 
 */ /* End of group ANALOG_IN_Test */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

