/**
 *******************************************************************************
 * @file    safety_cpu_pc.c
 * @brief   This file provides API functions for Safety Library - CPU Program Counter.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "safety_cpu_pc.h"


/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup CPUPC CPU PC
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CPUPC_Private_define CPUPC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group CPUPC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CPUPC_Private_define CPUPC Private Define
 *  @{
 */

// assume "mov r1, pc" is locatedt at the function entry address (even value)
// make "THUMB" address to "ARM" address then advance 4 for cortex M piplene implementation
#define GET_TAREGT_PC(x)        ((((uint32_t)(x)) & 0xfffffffe) + 4)
   
   
/**
 *  @}
 */ /* End of group CPUPC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CPUPC_Private_define CPUPC Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CPUPC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CPUPC_Private_typedef CPUPC Private Typedef
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_cpu1[81];
        char DEBUGMSG_cpu2[42];
        char DEBUGMSG_cpu3[58];
        char DEBUGMSG_cpu4[42];
        char DEBUGMSG_cpu5[58];
        char DEBUGMSG_cpu6[42];
        char DEBUGMSG_cpu7[58];
        char DEBUGMSG_cpu8[42];
        char DEBUGMSG_cpu9[58];
        char DEBUGMSG_cpu10[42];
        char DEBUGMSG_cpu11[58];
        char DEBUGMSG_cpu12[42];
        char DEBUGMSG_cpu13[58];
        char DEBUGMSG_cpu14[42];
        char DEBUGMSG_cpu15[58];
        char DEBUGMSG_cpu16[42];
        char DEBUGMSG_cpu17[58];
        char DEBUGMSG_cpu18[42];
        char DEBUGMSG_cpu19[58];
        char DEBUGMSG_cpu20[43];
    } debugmsg_t;
#endif

/**
 *  @}
 */ /* End of group CPUPC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CPUPC_Private_variables CPUPC Private Variables
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t cpu;
    
    char DEBUGMSG_cpu_temp[20][81] = {{"stored PC value 0xresult differs from estimated PC value 0xGET_TAREGT_PC(func1)."}, 
                                            {"jump to function address 0xfunc1 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc2 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc3 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc4 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc5 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc6 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc7 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc8 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc9 SUCCESS."},
                                            {"stored PC value 0xresult differs from estimated PC value."},
                                            {"jump to function address 0xfunc10 SUCCESS."}};
#endif
/**
 *  @}
 */ /* End of group CPUPC_Private_variables */
 
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CPUPC_Private_fuctions CPUPC Private Fuctions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief return PC value of function entry
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/

// in IAR compiler, the first instruction code must be "mov r1, pc" at the function entry address,
// and no stack operation exist at that point.
// note: the function entry address must be odd value because cortex-M always use THUMB instruction set.

// note: in Release build, we need to disable inline functions for func1() - func10()
static uint32_t func1(void);
static uint32_t func2(void);
static uint32_t func3(void);
static uint32_t func4(void);
static uint32_t func5(void);
static uint32_t func6(void);
static uint32_t func7(void);
static uint32_t func8(void);
static uint32_t func9(void);
static uint32_t func10(void);


#if defined ( __CC_ARM )        /* RealView Compiler */
static uint32_t func1(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func2(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func3(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func4(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func5(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func6(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func7(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func8(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func9(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

static uint32_t func10(void)
{
    uint32_t program_counter;
    __asm
    {
        mov program_counter, __current_pc()
    }

    return program_counter;
}

#elif defined ( __ICCARM__ )    /* IAR Compiler */

#pragma optimize=no_inline
static uint32_t func1(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func2(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func3(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func4(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func5(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func6(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func7(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func8(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func9(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}

#pragma optimize=no_inline
static uint32_t func10(void)
{
    uint32_t program_counter;

    asm("mov %0, pc" : "=r" (program_counter));

    return program_counter;
}
    
#endif

/**
 *  @}
 */ /* End of group CPUPC_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CPUPC_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     CPU Program Counter check
  * @retval    true  :Success.
  * @note      this function will never return if check failed.
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/
bool safety_CPU_ProgramCounter(void)
{
  uint32_t result;      // PC register values stored
 
  // jump to func1 and make sure PC is the same as estimated value
  result = func1();  
  if (result != GET_TAREGT_PC(func1)) {
    // this can occur if generated code is different from the development time 
    // or func1() is made to inline function
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value 0x%08x.\r\n", result, GET_TAREGT_PC(func1)));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu1,&DEBUGMSG_cpu_temp[0][0]);
#endif
    return false;
  }

#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func1));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu2,&DEBUGMSG_cpu_temp[1][0]);
#endif

  result = func2();  
  if (result != GET_TAREGT_PC(func2)) {
    // this can occur if generated code is different from the development time 
    
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu3,&DEBUGMSG_cpu_temp[2][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func2));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu4,&DEBUGMSG_cpu_temp[3][0]);
#endif
  
  result = func3();  
  if (result != GET_TAREGT_PC(func3)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu5,&DEBUGMSG_cpu_temp[4][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func3));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu6,&DEBUGMSG_cpu_temp[5][0]);
#endif

  result = func4();  
  if (result != GET_TAREGT_PC(func4)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu7,&DEBUGMSG_cpu_temp[6][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func4));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu8,&DEBUGMSG_cpu_temp[7][0]);
#endif

  result = func5();  
  if (result != GET_TAREGT_PC(func5)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu9,&DEBUGMSG_cpu_temp[8][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func5));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu10,&DEBUGMSG_cpu_temp[9][0]);
#endif

  result = func6();  
  if (result != GET_TAREGT_PC(func6)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu11,&DEBUGMSG_cpu_temp[10][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func6));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu12,&DEBUGMSG_cpu_temp[11][0]);
#endif

  result = func7();  
  if (result != GET_TAREGT_PC(func7)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu13,&DEBUGMSG_cpu_temp[12][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func7));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu14,&DEBUGMSG_cpu_temp[13][0]);
#endif

  result = func8();  
  if (result != GET_TAREGT_PC(func8)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu15,&DEBUGMSG_cpu_temp[14][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func8));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu16,&DEBUGMSG_cpu_temp[15][0]);
#endif

  result = func9();  
  if (result != GET_TAREGT_PC(func9)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu17,&DEBUGMSG_cpu_temp[16][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func9));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu18,&DEBUGMSG_cpu_temp[17][0]);
#endif
  
  result = func10();  
  if (result != GET_TAREGT_PC(func10)) {
    // this can occur if generated code is different from the development time 
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("stored PC value 0x%08x differs from estimated PC value.\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(cpu.DEBUGMSG_cpu19,&DEBUGMSG_cpu_temp[18][0]);
#endif
    return false;
  }
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("jump to function address 0x%08x SUCCESS.\r\n", (uint32_t)func10));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(cpu.DEBUGMSG_cpu20,&DEBUGMSG_cpu_temp[19][0]);
#endif
  
  return true;	// true == OK
}

/**
 *  @}
 */ /* End of group CPUPC_Exported_functions */

/**
 *  @} 
 */ /* End of group CPUPC */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

