/**
 *******************************************************************************
 * @file    safety_digital_io.c
 * @brief   This file provides API functions for Safety Library - DIGITAL IN/OUT.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "safety_digital_io.h"


// gpio driver definitions
#include "txz_gpio.h"  

// use bsp functions
#include "bsp.h"  

   
/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup DIGITAL_IO_Test DIGITALIO Test
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_define DIGITAL_IO_Test Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_define DIGITAL_IO_Test Private Define
 *  @{
 */
    
  
/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_define DIGITAL_IO_Test Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_typedef DIGITAL_IO_Test Private Typedef
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_digital1[29];
        char DEBUGMSG_digital2[31];
        char DEBUGMSG_digital3[41];
        char DEBUGMSG_digital4[35];
        char DEBUGMSG_digital5[31];
        char DEBUGMSG_digital6[47];
        char DEBUGMSG_digital7[35];
        char DEBUGMSG_digital8[31];
        char DEBUGMSG_digital9[47];
    } debugmsg_t;
#endif

/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_variables DIGITAL IO Test Private Variables
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t digital;
    
    char DEBUGMSG_digital_temp[9][47] = {{"safety_DIN_PE2: read failed."}, 
                                         {"safety_DOUT_PJ0: write failed."},
                                         {"safety_DigitalIO: gpio_func(PL4) failed."},
                                         {"safety_DigitalIO: write Hi failed."},
                                         {"safety_DigitalIO: Read failed."},
                                         {"safety_DigitalIO: Lo input detect. must be Hi."},
                                         {"safety_DigitalIO: write Lo failed."},
                                         {"safety_DigitalIO: read failed."},
                                         {"safety_DigitalIO: Hi input detect. must be Lo."}};
#endif
/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_fuctions DIGITAL_IO_Test Private Fuctions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DIGITAL_IO_Test_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Read PE2 Port
  * @retval    0 :Lo, 1 :Hi
  * @note      connected to USW1 switch in TMPM4K4-SBK
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/
uint32_t safety_DIN_PE2(void)
{
    gpio_t *p_gpio = bsp_get_gpio_instance_address();   // already initialized by BSP
    gpio_pinstate_t val = GPIO_PIN_RESET;
    
    if(gpio_read_bit(p_gpio, GPIO_PORT_E, GPIO_PORT_2, GPIO_Mode_DATA, &val)!= TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DIN_PE2: read failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital1,&DEBUGMSG_digital_temp[0][0]);
#endif
    }
  
    return ((val == GPIO_PIN_SET) ? (uint32_t)(1) : (uint32_t)(0));
}

/*--------------------------------------------------*/
/**
  * @brief     Write PJ0 Port
  * @retval    none
  * @note      connected to LED1 in TMPM4K4-SBK
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/
void safety_DOUT_PJ0(uint32_t output)
{
    gpio_t *p_gpio = bsp_get_gpio_instance_address();   // already initialized by BSP
	
    if (gpio_write_bit(p_gpio, GPIO_PORT_J, GPIO_PORT_0, GPIO_Mode_DATA, output) != TXZ_SUCCESS) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_DOUT_PJ0: write failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(digital.DEBUGMSG_digital2,&DEBUGMSG_digital_temp[1][0]);
#endif
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Digital Loopback Test
  * @retval    none
  * @note      connected to LED4 in TMPM4K4-SBK
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/

// use PL4 for Digital Loopback test
#define PORT_GROUP      GPIO_PORT_L
#define PORT_NUMBER     GPIO_PORT_4
#define PORT_FUNC       GPIO_PL4_INPUT  // this is actually the same as GPIO_PL4_OUTPUT

bool safety_DigitalIO(uint32_t *result_word)
{
    gpio_t *p_gpio = bsp_get_gpio_instance_address();   // already initialized by BSP
    gpio_pinstate_t input;
  
    // setup GPIO for Digital Loopback test (GPIO_PIN_INOUT)
    if (gpio_func(p_gpio, PORT_GROUP, PORT_NUMBER, PORT_FUNC, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: gpio_func(PL4) failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital3,&DEBUGMSG_digital_temp[2][0]);
#endif
    }

    // test for Hi output
    if (gpio_write_bit(p_gpio, PORT_GROUP, PORT_NUMBER, GPIO_Mode_DATA, GPIO_PIN_SET) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: write Hi failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital4,&DEBUGMSG_digital_temp[3][0]);
#endif
    }

    // check Hi input
    if(gpio_read_bit(p_gpio, PORT_GROUP, PORT_NUMBER, GPIO_Mode_DATA, &input)!= TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: Read failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital5,&DEBUGMSG_digital_temp[4][0]);
#endif
    }
    if (input != GPIO_PIN_SET)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: Lo input detect. must be Hi.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital6,&DEBUGMSG_digital_temp[5][0]);
#endif
      
      
      if (result_word != SAFETY_NULL) {
        *result_word = (1 << 0);       // set bit0 only
      }
      
      // anyway, test failed
      return false;
    }

    // test for Lo output
    if (gpio_write_bit(p_gpio, PORT_GROUP, PORT_NUMBER, GPIO_Mode_DATA, GPIO_PIN_RESET) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: write Lo failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital7,&DEBUGMSG_digital_temp[6][0]);
#endif
    }

    // check Lo input
    if(gpio_read_bit(p_gpio, PORT_GROUP, PORT_NUMBER, GPIO_Mode_DATA, &input) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: read failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital8,&DEBUGMSG_digital_temp[7][0]);
#endif
    }
    if (input != GPIO_PIN_RESET) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_DigitalIO: Hi input detect. must be Lo.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(digital.DEBUGMSG_digital9,&DEBUGMSG_digital_temp[8][0]);
#endif
      
      if (result_word != SAFETY_NULL) {
        *result_word = (1 << 1);       // set bit1 only
      }
      
      // anyway, test failed
      return false;
    }

    // otherwise, test ALL SUCCESS
    return true;
}

/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Exported_functions */

/**
 *  @} 
 */ /* End of group DIGITAL_IO_Test */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

