/**
 *******************************************************************************
 * @file    safety_ram.c
 * @brief   This file provides API functions for Safety Library - RAM.
 * @version V0.1.0.2
 * $Date:: 2019-02-14 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "safety_ram.h"


#if defined ( __CC_ARM )        /* RealView Compiler */

__ASM bool safety_RAM(uint32_t start, uint32_t length, uint32_t *result_word)
{
	/* bool safety_RAM(uint32_t start, uint32_t length, uint32_t *result_word) */
	/* r0 = start */
	/* r1 = length */
	/* r2 = ptr to result_word */

	push	{r4-r6}
	mov	r3, #0x55555555
	mov	r4, #0xaaaaaaaa

	/*  start &= ~0x3 */
	lsrs	r0, r0, #2
	lsls	r0, r0, #2

	/*  length &= ~0x3 */
	lsrs	r1, r1, #2
	lsls	r1, r1, #2

	/*  for (p = (uint32_t *)start; length > 0; p++, length -= 4) { */
	b	test1
}

__ASM void test0(void)
{
	/* restore */
	str	r5, [r0]
	
	/* next */
	adds	r0, r0, #4
	subs	r1, r1, #4
}

__ASM void test1(void)
{
	/* check length */
	cmp	r1, #0
	beq	ret_OK

	/*    temp = *p */
	ldr	r5, [r0]

	/* write 0x55555555 and check */
	str	r3, [r0]
	ldr	r6, [r0]
	cmp	r6, r3
	bne	test4
}

__ASM void test3(void)
{
	/* write 0xaaaaaaaa and check */
	str	r4, [r0]
	ldr	r6, [r0]
	cmp	r6, r4
	beq	test0
}

__ASM void test4(void)
{
	/* if (result_word != NULL) */
	cmp	r2, #0
	beq	test5

	/* *result_word = (uint32_t)p */
	str	r0, [r2]
}

__ASM void test5(void)
{
	/* *p = temp */
	/* return false */
	str	r5, [r0]
	movs	r0, #0
	b	test6
}
__ASM void ret_OK(void)
{
	/* return true */
	movs	r0, #1
}

__ASM void test6(void)
{
	pop	{r4-r6}
	bx	lr
}

#elif defined ( __ICCARM__ )    /* IAR Compiler */


bool safety_RAM(uint32_t start, uint32_t length, uint32_t *result_word)
{
	/* bool safety_RAM(uint32_t start, uint32_t length, uint32_t *result_word) */
	/* r0 = start */
	/* r1 = length */
	/* r2 = ptr to result_word */

	asm("push	{r4-r6}");
	asm("mov	r3, #0x55555555");
	asm("mov	r4, #0xaaaaaaaa");

	/*  start &= ~0x3 */
	asm("lsrs	r0, r0, #2");
	asm("lsls	r0, r0, #2");

	/*  length &= ~0x3 */
	asm("lsrs	r1, r1, #2");
	asm("lsls	r1, r1, #2");

	/*  for (p = (uint32_t *)start; length > 0; p++, length -= 4) { */
	asm("b	L1");
	
	return 0;  /* dummey */
}

void L0(void)
{
	/* restore */
	asm("str	r5, [r0]");
	
	/* next */
	asm("adds	r0, r0, #4");
	asm("subs	r1, r1, #4");
	asm("b L1");
}

void L1(void)
{
	/* check length */
	asm("cmp	r1, #0");
	asm("beq	ret_OK");

	/*    temp = *p */
	asm("ldr	r5, [r0]");

	/* write 0x55555555 and check */
	asm("str	r3, [r0]");
	asm("ldr	r6, [r0]");
	asm("cmp	r6, r3");
	asm("bne	L4");
	asm("b L3");
}

void L3(void)
{
	/* write 0xaaaaaaaa and check */
	asm("str	r4, [r0]");
	asm("ldr	r6, [r0]");
	asm("cmp	r6, r4");
	asm("beq	L0");
}

void L4(void)
{
	/* if (result_word != NULL) */
	asm("cmp	r2, #0");
	asm("beq	L5");

	/* *result_word = (uint32_t)p */
	asm("str	r0, [r2]");
}
void L5(void)
{
	/* *p = temp */
	/* return false */
	asm("str	r5, [r0]");
	asm("movs	r0, #0");
	asm("b	L6");
}

void ret_OK(void)
{
	/* return true */
	asm("movs	r0, #1");
}

void L6(void)
{
	asm("pop	{r4-r6}");
	asm("bx	lr");
}

#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */
