/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Analog In Sample Test Application.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup DIO_Sample
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "txz_sample_def.h"
#include "bsp.h"
//#include "bsp_sw.h"
#include "bsp_led.h"

// safety tests minimum definition (for "bool" and "SAFETY_NULL")
#include "txz_safety_def.h"

// use Analog In safety library
#include "safety_analog_in.h"

/*------------------------------------------------------------------------------*/
/*  Externals                                                                   */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    gpio_t      gpio_obj;            /*!< Application :GPIO. */
} instance_t;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_main1[54];
        char DEBUGMSG_main2[25];
        char DEBUGMSG_main3[24];
        char DEBUGMSG_main4[43];
        char DEBUGMSG_main5[9];
    } debugmsg_t;
#endif
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static instance_t instance; /*!< Instance.                    */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t debugmain;
    char DEBUGMSG_main_temp[5][54] = {{"Analog IN Test: FAILED. (result_word = 0xresult_word)"}, 
                                      {"Analog IN Test: SUCCESS."}, 
                                      {"safety_AIN_PE4: failed."}, 
                                      {"safety_AIN_PE4: SUCCESS. value=0xadc_value"},
                                      {"Finish!."}};
#endif
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }

    return (result);
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    bool result;
    uint32_t result_word;
    uint32_t adc_value;
    
    int result_code = 0;        // no error
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/

    // Self Check ADC using PE0/AINA10
    result = safety_AnalogIn(&result_word);
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("Analog IN Test: FAILED. (result_word = 0x%04x).\r\n", result_word));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main1,&DEBUGMSG_main_temp[0][0]);
#endif
      result_code = 1;
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("Analog IN Test: SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main2,&DEBUGMSG_main_temp[1][0]);
#endif
    }
    
    // ADC PE4/AIN14 (VR1)
    result = safety_AIN_PE4(&adc_value);
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_AIN_PE4: failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(debugmain.DEBUGMSG_main3,&DEBUGMSG_main_temp[2][0]);
#endif
      result_code = 2;
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
       DEBUGMSG(1, ("safety_AIN_PE4: SUCCESS. value=0x%x\r\n", adc_value));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
       strcpy(debugmain.DEBUGMSG_main4,&DEBUGMSG_main_temp[3][0]);
#endif
   }

test_done:  

    {
      led_t *p_led0 = &instance.led[0];
      led_t *p_led1 = &instance.led[1];
      led_t *p_led2 = &instance.led[2];

      // report result
      // LED0 = SUCCESS, LED1 = ANALOG Test FAIL, LED2 = Read ADC FAIL
      switch (result_code) {
      case 0:     //  ALL SUCCESS
        led_turn_on(p_led0);
        break;
        
      case 1:     // ANALOG test FAILED
        led_turn_on(p_led1);
        break;
        
      case 2:     // Read ADC test FAILED
        led_turn_on(p_led2);
        break;
        
      default:
        // INTERNAL ERROR?
        break;
      }
    }
    
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("Finish!.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(debugmain.DEBUGMSG_main5,&DEBUGMSG_main_temp[4][0]);
#endif
    
    /* dummy loop */
    while (1){
        __NOP();
    }
        
}

/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_counter(BSPCounter cnt)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(uint32_t ch)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rmc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RAM Parity IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_parity(void)
{
    /* Sample Soft don't use this function. */
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @} 
 */ /* End of group DIO_Sample */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
