/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Digital I/O Test Sample Application.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup DIO_Sample
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "txz_sample_def.h"
#include "bsp.h"

// use hal.c to determine timeout
#include "txz_hal.h"
   
// safety tests minimum definition (for "bool" and "SAFETY_NULL")
#include "txz_safety_def.h"

// use Digital I/O safety library
#include "safety_digital_io.h"

/*------------------------------------------------------------------------------*/
/*  Externals                                                                   */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_main1[30];
        char DEBUGMSG_main2[20];
        char DEBUGMSG_main3[9];
    } debugmsg_t;
#endif
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t debugmain;
    char DEBUGMSG_main_temp[3][30] = {{"DigitalIO: Failed. (0xresult)"}, 
                                      {"DigitalIO: SUCCESS."}, 
                                      {"Finish!."}};
#endif
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
   
    // enable SysTick Interrupt (for TickCount)
    {
      // start SysTick IRQ
      uint32_t period = 80000;  // 80MHz / 1000Hz = 1msec SysTick
     
      if (SysTick_Config(period))    /* systick interrupt cycle setting */
      {
      }
      
      // no need this
      //NVIC_EnableIRQ(SysTick_IRQn);   /* systick interrupt enable */
    }
       
    return (result);
}


void Sleep(uint32_t msec)
{
    uint32_t old_tick = hal_get_tick();
    
    while ((hal_get_tick() - old_tick) < msec) {	
      // do nothing
      __NOP();
    }
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t result;
  
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    
    // Digital Loopback by PL4
    if (!safety_DigitalIO(&result)) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("DigitalIO: Failed. (0x%04x)\r\n", result));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main1,&DEBUGMSG_main_temp[0][0]);
#endif
      // in case test FAILED: no LED lit.
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("DigitalIO: SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main2,&DEBUGMSG_main_temp[1][0]);
#endif
    }    

    // forever loop
    // copy PE2 input to PJ0 output
    // note: USW1 push switch input is inverted (off = 1, on = 0)
    for (;;) {
      uint32_t sw;
      
      // input USW1 status
      sw = safety_DIN_PE2();
      
      //DEBUGMSG(1, ("[%d]", sw));

      // output LED1
      safety_DOUT_PJ0(sw);
      
      // wait 100msec
      Sleep(100);
      
    }

    //
    // never reach here...
    //
    
test_done:
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("Finish!.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(debugmain.DEBUGMSG_main3,&DEBUGMSG_main_temp[2][0]);
#endif
    
    /* dummy loop */
    while (1){
        __NOP();
    }
    
}

/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    // use "hal.c" to get timeout
    hal_inc_tick();
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_counter(BSPCounter cnt)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(uint32_t ch)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rmc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RAM Parity IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_parity(void)
{
    /* Sample Soft don't use this function. */
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @} 
 */ /* End of group DIO_Sample */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
