/** 
 *******************************************************************************
 * @file    main.c
 * @brief   MEMORY Sample Test Application.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup MEMORY_Sample
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#if defined ( __ICCARM__ )    /* IAR Compiler */
    #include "intrinsics.h"
#endif
#include "txz_sample_def.h"
#include "bsp.h"
//#include "bspp_sw.h"
#include "bsp_led.h"
   
// safety tests minimum definition (for "bool" and "SAFETY_NULL")
#include "txz_safety_def.h"

// use CPU safety library
#include "safety_ram.h"
#include "safety_flash.h"

/*------------------------------------------------------------------------------*/
/*  Externals                                                                   */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    gpio_t      gpio_obj;            /*!< Application :GPIO. */
} instance_t;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_main1[49];
        char DEBUGMSG_main2[26];
        char DEBUGMSG_main3[45];
        char DEBUGMSG_main4[22];
        char DEBUGMSG_main5[94];
        char DEBUGMSG_main6[15];
        char DEBUGMSG_main7[33];
        char DEBUGMSG_main8[9];
    } debugmsg_t;
#endif
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
uint32_t testtesttest;
static instance_t instance; /*!< Instance.                    */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t debugmain;
    char DEBUGMSG_main_temp[8][94] = {{"RAM (Non Stack): Failed! (Address=0xresult_word)"}, 
                                      {"RAM (Non Stack): SUCCESS."}, 
                                      {"RAM (Stack): Failed! (Address=0xresult_word)"},
                                      {"RAM (Stack): SUCCESS."},
                                      {"__checksum_begin=0x__checksum_begin, __checksum_end=0x__checksum_end, __checksum=0x__checksum"},
                                      {"FLASH: Failed!"},
                                      {"FLASH: SUCCESS. CRC=0x__checksum"},
                                      {"Finish!."}};
                                      
    uint16_t CRC16Table[256] = 
            {
            0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF,
            0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE,
            0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE, 0xF5CF, 0xC5AC, 0xD58D,
            0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D, 0xC7BC,
            0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B,
            
            0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,0xDBFD, 0xCBDC, 0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A,
            0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD, 0xAD2A, 0xBD0B, 0x8D68, 0x9D49,
            0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A, 0x9F59, 0x8F78,
            0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
            0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
            
            0xB5EA, 0xA5CB, 0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
            0xA7DB, 0xB7FA, 0x8799, 0x97B8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
            0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9, 0xB98A, 0xA9AB,0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
            0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
            0xFD2E, 0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9,0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
            
            0xEF1F, 0xFF3E, 0xCF5D, 0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8,0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
            };

    uint32_t CRC32Table[256] = 
            {
            0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9, 0x130476dc, 0x17c56b6b, 0x1a864db2, 0x1e475005, 0x2608edb8, 0x22c9f00f, 0x2f8ad6d6, 0x2b4bcb61, 0x350c9b64, 0x31cd86d3, 0x3c8ea00a, 0x384fbdbd, 
            0x4c11db70, 0x48d0c6c7, 0x4593e01e, 0x4152fda9, 0x5f15adac, 0x5bd4b01b, 0x569796c2, 0x52568b75, 0x6a1936c8, 0x6ed82b7f, 0x639b0da6, 0x675a1011, 0x791d4014, 0x7ddc5da3, 0x709f7b7a, 0x745e66cd, 
            0x9823b6e0, 0x9ce2ab57, 0x91a18d8e, 0x95609039, 0x8b27c03c, 0x8fe6dd8b, 0x82a5fb52, 0x8664e6e5, 0xbe2b5b58, 0xbaea46ef, 0xb7a96036, 0xb3687d81, 0xad2f2d84, 0xa9ee3033, 0xa4ad16ea, 0xa06c0b5d, 
            0xd4326d90, 0xd0f37027, 0xddb056fe, 0xd9714b49, 0xc7361b4c, 0xc3f706fb, 0xceb42022, 0xca753d95, 0xf23a8028, 0xf6fb9d9f, 0xfbb8bb46, 0xff79a6f1, 0xe13ef6f4, 0xe5ffeb43, 0xe8bccd9a, 0xec7dd02d, 
            0x34867077, 0x30476dc0, 0x3d044b19, 0x39c556ae, 0x278206ab, 0x23431b1c, 0x2e003dc5, 0x2ac12072, 0x128e9dcf, 0x164f8078, 0x1b0ca6a1, 0x1fcdbb16, 0x018aeb13, 0x054bf6a4, 0x0808d07d, 0x0cc9cdca, 
            
            0x7897ab07, 0x7c56b6b0, 0x71159069, 0x75d48dde, 0x6b93dddb, 0x6f52c06c, 0x6211e6b5, 0x66d0fb02, 0x5e9f46bf, 0x5a5e5b08, 0x571d7dd1, 0x53dc6066, 0x4d9b3063, 0x495a2dd4, 0x44190b0d, 0x40d816ba, 
            0xaca5c697, 0xa864db20, 0xa527fdf9, 0xa1e6e04e, 0xbfa1b04b, 0xbb60adfc, 0xb6238b25, 0xb2e29692, 0x8aad2b2f, 0x8e6c3698, 0x832f1041, 0x87ee0df6, 0x99a95df3, 0x9d684044, 0x902b669d, 0x94ea7b2a, 
            0xe0b41de7, 0xe4750050, 0xe9362689, 0xedf73b3e, 0xf3b06b3b, 0xf771768c, 0xfa325055, 0xfef34de2, 0xc6bcf05f, 0xc27dede8, 0xcf3ecb31, 0xcbffd686, 0xd5b88683, 0xd1799b34, 0xdc3abded, 0xd8fba05a, 
            0x690ce0ee, 0x6dcdfd59, 0x608edb80, 0x644fc637, 0x7a089632, 0x7ec98b85, 0x738aad5c, 0x774bb0eb, 0x4f040d56, 0x4bc510e1, 0x46863638, 0x42472b8f, 0x5c007b8a, 0x58c1663d, 0x558240e4, 0x51435d53, 
            0x251d3b9e, 0x21dc2629, 0x2c9f00f0, 0x285e1d47, 0x36194d42, 0x32d850f5, 0x3f9b762c, 0x3b5a6b9b, 0x0315d626, 0x07d4cb91, 0x0a97ed48, 0x0e56f0ff, 0x1011a0fa, 0x14d0bd4d, 0x19939b94, 0x1d528623, 
            
            0xf12f560e, 0xf5ee4bb9, 0xf8ad6d60, 0xfc6c70d7, 0xe22b20d2, 0xe6ea3d65, 0xeba91bbc, 0xef68060b, 0xd727bbb6, 0xd3e6a601, 0xdea580d8, 0xda649d6f, 0xc423cd6a, 0xc0e2d0dd, 0xcda1f604, 0xc960ebb3, 
            0xbd3e8d7e, 0xb9ff90c9, 0xb4bcb610, 0xb07daba7, 0xae3afba2, 0xaafbe615, 0xa7b8c0cc, 0xa379dd7b, 0x9b3660c6, 0x9ff77d71, 0x92b45ba8, 0x9675461f, 0x8832161a, 0x8cf30bad, 0x81b02d74, 0x857130c3, 
            0x5d8a9099, 0x594b8d2e, 0x5408abf7, 0x50c9b640, 0x4e8ee645, 0x4a4ffbf2, 0x470cdd2b, 0x43cdc09c, 0x7b827d21, 0x7f436096, 0x7200464f, 0x76c15bf8, 0x68860bfd, 0x6c47164a, 0x61043093, 0x65c52d24, 
            0x119b4be9, 0x155a565e, 0x18197087, 0x1cd86d30, 0x029f3d35, 0x065e2082, 0x0b1d065b, 0x0fdc1bec, 0x3793a651, 0x3352bbe6, 0x3e119d3f, 0x3ad08088, 0x2497d08d, 0x2056cd3a, 0x2d15ebe3, 0x29d4f654, 
            0xc5a92679, 0xc1683bce, 0xcc2b1d17, 0xc8ea00a0, 0xd6ad50a5, 0xd26c4d12, 0xdf2f6bcb, 0xdbee767c, 0xe3a1cbc1, 0xe760d676, 0xea23f0af, 0xeee2ed18, 0xf0a5bd1d, 0xf464a0aa, 0xf9278673, 0xfde69bc4, 
            
            0x89b8fd09, 0x8d79e0be, 0x803ac667, 0x84fbdbd0, 0x9abc8bd5, 0x9e7d9662, 0x933eb0bb, 0x97ffad0c, 0xafb010b1, 0xab710d06, 0xa6322bdf, 0xa2f33668, 0xbcb4666d, 0xb8757bda, 0xb5365d03, 0xb1f740b4
            };
#endif
//
// checksum related symbols - use Embedded Workbench linker option and checksum tab
// then following externals are generated by linker.
//

#if defined ( __ICCARM__ )    /* IAR Compiler */
    // use IAR Linker defined symbol "__checksum" 
    extern uint32_t __checksum;             // CRC32 for 0 - 0x7fff except CRC32 itself

    // use IAR Linker defined symbol "__checksum_begin", "__checksum_end" 
    // following symbols are defined as assembler LABEL
    extern uint8_t __checksum_begin;       // address of __checksum_begin == 0x0
    extern uint8_t __checksum_end;         // address of __checksum_end == 0x7fff or lower
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    // use IAR Linker defined symbol "__checksum" 
    uint32_t __checksum;             // CRC32 for 0 - 0x7fff except CRC32 itself

    // use IAR Linker defined symbol "__checksum_begin", "__checksum_end" 
    // following symbols are defined as assembler LABEL
    uint8_t __checksum_begin;       // address of __checksum_begin == 0x0
    uint8_t __checksum_end;         // address of __checksum_end == 0x7fff or lower
#endif


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */

#if defined ( __ICCARM__ )    /* IAR Compiler */
    // TMPM4K4A: 18KB RAM staring at 0x20000000, size 0x4800
    #pragma section = "CSTACK"

    // stack area
    #define RAM_TEST_START2         ((uint32_t)__section_begin("CSTACK"))
    #define RAM_TEST_SIZE2          ((uint32_t)__section_size("CSTACK"))

    // non stack area   
    #define RAM_TEST_START1         ((uint32_t)RAM_BASE)
    #define RAM_TEST_SIZE1          ((uint32_t)(__section_begin("CSTACK")) - (uint32_t)RAM_BASE)

    // in IAR we can use Linker Symbol so use it
    #define FLASH_TEST_START        (uint32_t)(&__checksum_begin)
    #define FLASH_TEST_SIZE         (uint32_t)(&__checksum_end - &__checksum_begin + 1)
    #define FLASH_TEST_CRC32VALUE   (__checksum)

#elif defined ( __CC_ARM   )      /* RealView Compiler */
    // stack area
    extern uint32_t Image$$ARM_LIB_STACK$$Base;
    extern uint32_t Image$$ARM_LIB_STACK$$Length;
    
    #define RAM_TEST_START2         (uint32_t)&Image$$ARM_LIB_STACK$$Base
    #define RAM_TEST_SIZE2          (uint32_t)&Image$$ARM_LIB_STACK$$Length

    // non stack area
    #define RAM_TEST_START1         ((uint32_t)RAM_BASE)
    #define RAM_TEST_SIZE1          ((uint32_t)&Image$$ARM_LIB_STACK$$Base - (uint32_t)RAM_BASE)

    // in IAR we can use Linker Symbol so use it
    #define FLASH_TEST_START        (uint32_t)(0x00000000)
    #define FLASH_TEST_SIZE         (uint32_t)(0x00001a27 - FLASH_TEST_START + 1)
    #define FLASH_TEST_CRC32VALUE   (__checksum)
#endif

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }

    return (result);
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/

/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    bool result;
    uint32_t result_word;
    
    int result_code = 0;        // no error
    
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- application ---*/
    // this will turn off all LEDs
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/

    // RAM Test for NON STACK area
    result = safety_RAM(RAM_TEST_START1, RAM_TEST_SIZE1, &result_word);
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("RAM (Non Stack): Failed! (Address=0x%08x)\r\n", result_word));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main1,&DEBUGMSG_main_temp[0][0]);
#endif
      result_code = 1;
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("RAM (Non Stack): SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main2,&DEBUGMSG_main_temp[1][0]);
#endif
    }
    

    // RAM Test for STACK area
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        __istate_t old_state = __get_interrupt_state(); 
        __disable_interrupt();
        result = safety_RAM(RAM_TEST_START2, RAM_TEST_SIZE2, &result_word);
        __set_interrupt_state(old_state);
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        uint32_t primask_data;
        __asm
        {
            /* __istate_t old_state = __get_interrupt_state();  */
            mrs primask_data, primask
            
            /* __disable_interrupt(); */
            cpsid i
        }
        
        result = safety_RAM(RAM_TEST_START2, RAM_TEST_SIZE2, &result_word);
            
        __asm
        {
            /* __set_interrupt_state(old_state); */
            msr primask, primask_data
        }
#endif
    }
    
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("RAM (Stack): Failed! (Address=0x%08x)\r\n", result_word));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main3,&DEBUGMSG_main_temp[2][0]);
#endif
      result_code = 1;
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("RAM (Stack): SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main4,&DEBUGMSG_main_temp[3][0]);
#endif
    }

    
    // FLASH Test
    // the test range includes CRC32 value area so total value made fixed value
#if defined ( __ICCARM__ )    /*IAR Compiler */
  #if CRC == CRC32
    DEBUGMSG(1, ("__checksum_begin=0x%08x, __checksum_end=0x%08x, __checksum=0x%08x\r\n",
           (uint32_t)&__checksum_begin, (uint32_t)&__checksum_end, __checksum));
  #endif
  
  #if CRC == CRC16
    DEBUGMSG(1, ("__checksum_begin=0x%08x, __checksum_end=0x%08x, __checksum=0x%08x\r\n",
           (uint32_t)&__checksum_begin, (uint32_t)&__checksum_end, (uint16_t)__checksum));
  #endif
    result = safety_FLASH(FLASH_TEST_START, FLASH_TEST_SIZE, FLASH_TEST_CRC32VALUE);
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(debugmain.DEBUGMSG_main5,&DEBUGMSG_main_temp[4][0]);

    uint32_t i;

#if CRC == CRC32
    __checksum = 0 ^ 0xFFFFFFFF;

    for(i=0; i<FLASH_TEST_SIZE; i++){
        __checksum = (__checksum << 8) ^ (CRC32Table[(__checksum >> 24) ^ ((*(uint32_t *)(FLASH_TEST_START + i)) & 0xFF)]);
    }
#endif

#if CRC == CRC16
    __checksum = 0xFFFF;

    for(i=0; i<FLASH_TEST_SIZE; i++){
        __checksum = (uint16_t)((__checksum << 8) ^ (CRC16Table[(__checksum >> 8) ^ ((*(uint32_t *)(FLASH_TEST_START + i)) & 0xFF)]));
    }
  
#endif
    result = safety_FLASH(FLASH_TEST_START, FLASH_TEST_SIZE, FLASH_TEST_CRC32VALUE);
#endif
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("FLASH: Failed!\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main6,&DEBUGMSG_main_temp[5][0]);
#endif
      result_code = 2;
      goto test_done;
    }
    else {
      // make "__checksum" reference here. otherwise Linker will fail.
#if defined ( __ICCARM__ )    /*IAR Compiler */
#if CRC == CRC32
        DEBUGMSG(1, ("FLASH: SUCCESS. CRC=0x%08x\r\n", __checksum));
#endif
#if CRC == CRC16
        DEBUGMSG(1, ("FLASH: SUCCESS. CRC=0x%08x\r\n", (uint16_t)__checksum));
#endif
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main7,&DEBUGMSG_main_temp[6][0]);
#endif
        result_code = 0;
    }

test_done:  
  
    {
      led_t *p_led0 = &instance.led[0];
      led_t *p_led1 = &instance.led[1];
      led_t *p_led2 = &instance.led[2];

      // report result
      // LED0 = SUCCESS, LED1 = RAM FAIL, LED2 = FLASH FAIL
      switch (result_code) {
      case 0:     //  ALL SUCCESS
        led_turn_on(p_led0);
        break;
        
      case 1:     // RAM test FAILED
        led_turn_on(p_led1);
        break;
        
      case 2:     // FLASH test FAILED
        led_turn_on(p_led2);
        break;
        
      default:
        // INTERNAL ERROR?
        break;
      }
    }
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("Finish!.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main8,&DEBUGMSG_main_temp[7][0]);
#endif
    
    /* dummy loop */
    while (1){
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_counter(BSPCounter cnt)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(uint32_t ch)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rmc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RAM Parity IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_parity(void)
{
    /* Sample Soft don't use this function. */
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @} 
 */ /* End of group MEMORY_Sample */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
